@@ -1,6 +1,6 @@
 /* The MIT License
 
-   Copyright (c) 20082-2012 by Heng Li <lh3@me.com>
+   Copyright (c) 2008-2016 Broad Institute
 
    Permission is hereby granted, free of charge, to any person obtaining
    a copy of this software and associated documentation files (the
@@ -57,9 +57,10 @@ reghash_t *stk_reg_read(const char *fn)
 	int dret;
 	kstring_t *str;
 	// read the list
-	str = calloc(1, sizeof(kstring_t));
 	fp = strcmp(fn, "-")? gzopen(fn, "r") : gzdopen(fileno(stdin), "r");
+	if (fp == 0) return 0;
 	ks = ks_init(fp);
+	str = calloc(1, sizeof(kstring_t));
 	while (ks_getuntil(ks, 0, str, &dret) >= 0) {
 		int beg = -1, end = -1;
 		reglist_t *p;
@@ -298,10 +299,15 @@ int stk_trimfq(int argc, char *argv[])
 		fprintf(stderr, "         -b INT      trim INT bp from left (non-zero to disable -q/-l) [0]\n");
 		fprintf(stderr, "         -e INT      trim INT bp from right (non-zero to disable -q/-l) [0]\n");
 		fprintf(stderr, "         -L INT      retain at most INT bp from the 5'-end (non-zero to disable -q/-l) [0]\n");
+		fprintf(stderr, "         -Q          force FASTQ output\n");
 		fprintf(stderr, "\n");
 		return 1;
 	}
 	fp = strcmp(argv[optind], "-")? gzopen(argv[optind], "r") : gzdopen(fileno(stdin), "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	for (i = 0; i < 128; ++i)
 		q_int2real[i] = pow(10., -(i - 33) / 10.);
@@ -336,12 +342,12 @@ int stk_trimfq(int argc, char *argv[])
 				end = beg + min_len;
 			}
 		} else beg = 0, end = seq->seq.l;
-		putchar(seq->qual.l? '@' : '>'); fputs(seq->name.s, stdout); 
+		putchar(seq->is_fastq? '@' : '>'); fputs(seq->name.s, stdout); 
 		if (seq->comment.l) {
 			putchar(' '); puts(seq->comment.s);
 		} else putchar('\n');
 		fwrite(seq->seq.s + beg, 1, end - beg, stdout); putchar('\n');
-		if (seq->qual.l) {
+		if (seq->is_fastq) {
 			puts("+");
 			fwrite(seq->qual.s + beg, 1, end - beg, stdout); putchar('\n');
 		}
@@ -372,6 +378,10 @@ int stk_comp(int argc, char *argv[])
 		return 1;
 	}
 	fp = optind < argc && strcmp(argv[optind], "-")? gzopen(argv[optind], "r") : gzdopen(fileno(stdin), "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	dummy.n= dummy.m = 1; dummy.a = calloc(1, 8);
 	while ((l = kseq_read(seq)) >= 0) {
@@ -437,6 +447,10 @@ int stk_randbase(int argc, char *argv[])
 		return 1;
 	}
 	fp = (strcmp(argv[1], "-") == 0)? gzdopen(fileno(stdin), "r") : gzopen(argv[1], "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	while ((l = kseq_read(seq)) >= 0) {
 		int i;
@@ -489,6 +503,10 @@ int stk_hety(int argc, char *argv[])
 		}
 	}
 	fp = (strcmp(argv[optind], "-") == 0)? gzdopen(fileno(stdin), "r") : gzopen(argv[optind], "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	win_step = win_size / n_start;
 	buf = calloc(win_size, 1);
@@ -547,8 +565,16 @@ int stk_subseq(int argc, char *argv[])
 		return 1;
 	}
 	h = stk_reg_read(argv[optind+1]);
+	if (h == 0) {
+		fprintf(stderr, "[E::%s] failed to read the list of regions in file '%s'\n", __func__, argv[optind+1]);
+		return 1;
+	}
 	// subseq
 	fp = strcmp(argv[optind], "-")? gzopen(argv[optind], "r") : gzdopen(fileno(stdin), "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	while ((l = kseq_read(seq)) >= 0) {
 		reglist_t *p;
@@ -626,6 +652,10 @@ int stk_mergefa(int argc, char *argv[])
 		fp[i] = strcmp(argv[optind+i], "-")? gzopen(argv[optind+i], "r") : gzdopen(fileno(stdin), "r");
 		seq[i] = kseq_init(fp[i]);
 	}
+	if (fp[0] == 0 || fp[1] == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	cnt[0] = cnt[1] = cnt[2] = cnt[3] = cnt[4] = 0;
 	srand48(11);
 	while (kseq_read(seq[0]) >= 0) {
@@ -707,6 +737,10 @@ int stk_famask(int argc, char *argv[])
 		fp[i] = strcmp(argv[optind+i], "-")? gzopen(argv[optind+i], "r") : gzdopen(fileno(stdin), "r");
 		seq[i] = kseq_init(fp[i]);
 	}
+	if (fp[0] == 0 || fp[1] == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	while (kseq_read(seq[0]) >= 0) {
 		int min_l, c[2];
 		kseq_read(seq[1]);
@@ -746,6 +780,10 @@ int stk_mutfa(int argc, char *argv[])
 	// read the list
 	str = calloc(1, sizeof(kstring_t));
 	fp = strcmp(argv[2], "-")? gzopen(argv[2], "r") : gzdopen(fileno(stdin), "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	ks = ks_init(fp);
 	while (ks_getuntil(ks, 0, str, &dret) >= 0) {
 		char *s = strdup(str->s);
@@ -775,6 +813,10 @@ int stk_mutfa(int argc, char *argv[])
 	free(str->s); free(str);
 	// mutfa
 	fp = strcmp(argv[1], "-")? gzopen(argv[1], "r") : gzdopen(fileno(stdin), "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	while ((l = kseq_read(seq)) >= 0) {
 		reglist_t *p;
@@ -817,6 +859,10 @@ int stk_listhet(int argc, char *argv[])
 		return 1;
 	}
 	fp = (strcmp(argv[1], "-") == 0)? gzdopen(fileno(stdin), "r") : gzopen(argv[1], "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	while ((l = kseq_read(seq)) >= 0) {
 		for (i = 0; i < l; ++i) {
@@ -902,6 +948,10 @@ int stk_cutN(int argc, char *argv[])
 		return 1;
 	}
 	fp = (strcmp(argv[optind], "-") == 0)? gzdopen(fileno(stdin), "r") : gzopen(argv[optind], "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	ks = kseq_init(fp);
 	while ((l = kseq_read(ks)) >= 0) {
 		int k = 0, begin = 0, end = 0;
@@ -930,6 +980,10 @@ int stk_hrun(int argc, char *argv[])
 	}
 	if (argc == optind + 2) min_len = atoi(argv[optind+1]);
 	fp = (strcmp(argv[optind], "-") == 0)? gzdopen(fileno(stdin), "r") : gzopen(argv[optind], "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	ks = kseq_init(fp);
 	while (kseq_read(ks) >= 0) {
 		c = ks->seq.s[0]; l = 1; beg = 0;
@@ -989,7 +1043,7 @@ int stk_sample(int argc, char *argv[])
 		return 1;
 	}
 	frac = atof(argv[optind+1]);
-	if (frac > 1.) num = (uint64_t)(frac + .499), frac = 0.;
+	if (frac >= 1.0) num = (uint64_t)(frac + .499), frac = 0.;
 	else if (twopass) {
 		fprintf(stderr, "[W::%s] when sampling a fraction, option -2 is ignored.", __func__);
 		twopass = 0;
@@ -1006,6 +1060,10 @@ int stk_sample(int argc, char *argv[])
 		}
 
 		fp = strcmp(argv[optind], "-")? gzopen(argv[optind], "r") : gzdopen(fileno(stdin), "r");
+		if (fp == 0) {
+			fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+			return 1;
+		}
 		seq = kseq_init(fp);
 		n_seqs = 0;
 		while (kseq_read(seq) >= 0) {
@@ -1037,6 +1095,10 @@ int stk_sample(int argc, char *argv[])
 		buf = malloc(num * 8);
 		for (i = 0; i < num; ++i) buf[i] = UINT64_MAX;
 		fp = gzopen(argv[optind], "r");
+		if (fp == 0) {
+			fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+			return 1;
+		}
 		seq = kseq_init(fp);
 		n_seqs = 0;
 		while (kseq_read(seq) >= 0) {
@@ -1117,14 +1179,14 @@ int stk_seq(int argc, char *argv[])
 {
 	gzFile fp;
 	kseq_t *seq;
-	int c, qual_thres = 0, flag = 0, qual_shift = 33, mask_chr = 0, min_len = 0, max_q = 255;
+	int c, qual_thres = 0, flag = 0, qual_shift = 33, mask_chr = 0, min_len = 0, max_q = 255, fake_qual = -1;
 	unsigned i, line_len = 0;
 	int64_t n_seqs = 0;
 	double frac = 1.;
 	khash_t(reg) *h = 0;
 	krand_t *kr = 0;
 
-	while ((c = getopt(argc, argv, "N12q:l:Q:aACrn:s:f:M:L:cVUX:S")) >= 0) {
+	while ((c = getopt(argc, argv, "N12q:l:Q:aACrn:s:f:M:L:cVUX:SF:")) >= 0) {
 		switch (c) {
 			case 'a':
 			case 'A': flag |= 1; break;
@@ -1146,6 +1208,7 @@ int stk_seq(int argc, char *argv[])
 			case 'L': min_len = atoi(optarg); break;
 			case 's': kr = kr_srand(atol(optarg)); break;
 			case 'f': frac = atof(optarg); break;
+			case 'F': fake_qual = *optarg; break;
 		}
 	}
 	if (kr == 0) kr = kr_srand(11);
@@ -1161,6 +1224,7 @@ int stk_seq(int argc, char *argv[])
 		fprintf(stderr, "         -f FLOAT  sample FLOAT fraction of sequences [1]\n");
 		fprintf(stderr, "         -M FILE   mask regions in BED or name list FILE [null]\n");
 		fprintf(stderr, "         -L INT    drop sequences with length shorter than INT [0]\n");
+		fprintf(stderr, "         -F CHAR   fake FASTQ quality []\n");
 		fprintf(stderr, "         -c        mask complement region (effective with -M)\n");
 		fprintf(stderr, "         -r        reverse complement\n");
 		fprintf(stderr, "         -A        force FASTA output (discard quality)\n");
@@ -1177,6 +1241,10 @@ int stk_seq(int argc, char *argv[])
 	}
 	if (line_len == 0) line_len = UINT_MAX;
 	fp = optind < argc && strcmp(argv[optind], "-")? gzopen(argv[optind], "r") : gzdopen(fileno(stdin), "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	qual_thres += qual_shift;
 	while (kseq_read(seq) >= 0) {
@@ -1215,6 +1283,15 @@ int stk_seq(int argc, char *argv[])
 			for (i = 0; i < seq->seq.l; ++i)
 				seq->seq.s[i] = toupper(seq->seq.s[i]);
 		if (flag & 1) seq->qual.l = 0; // option -a: fastq -> fasta
+		else if (fake_qual >= 33 && fake_qual <= 127) {
+			if (seq->qual.m < seq->seq.m) {
+				seq->qual.m = seq->seq.m;
+				seq->qual.s = (char*)realloc(seq->qual.s, seq->qual.m);
+			}
+			seq->qual.l = seq->seq.l;
+			memset(seq->qual.s, fake_qual, seq->qual.l);
+			seq->qual.s[seq->qual.l] = 0;
+		}
 		if (flag & 2) seq->comment.l = 0; // option -C: drop fasta/q comments
 		if (h) stk_mask(seq, h, flag&8, mask_chr); // masking
 		if (flag & 4) { // option -r: reverse complement
@@ -1274,6 +1351,10 @@ int stk_gc(int argc, char *argv[])
 	q = (1.0f - frac) / frac;
 
 	fp = strcmp(argv[optind], "-")? gzopen(argv[optind], "r") : gzdopen(fileno(stdin), "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	while (kseq_read(seq) >= 0) {
 		int i, start = 0, max_i = 0, n_hits = 0, start_hits = 0, max_hits = 0;
@@ -1317,6 +1398,10 @@ int stk_mergepe(int argc, char *argv[])
 	}
 	fp1 = strcmp(argv[1], "-")? gzopen(argv[1], "r") : gzdopen(fileno(stdin), "r");
 	fp2 = strcmp(argv[2], "-")? gzopen(argv[2], "r") : gzdopen(fileno(stdin), "r");
+	if (fp1 == 0 || fp2 == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq[0] = kseq_init(fp1);
 	seq[1] = kseq_init(fp2);
 	while (kseq_read(seq[0]) >= 0) {
@@ -1344,6 +1429,10 @@ int stk_dropse(int argc, char *argv[])
 		return 1;
 	}
 	fp = argc > 1 && strcmp(argv[1], "-")? gzopen(argv[1], "r") : gzdopen(fileno(stdin), "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 
 	memset(&last, 0, sizeof(kseq_t));
@@ -1381,6 +1470,10 @@ int stk_rename(int argc, char *argv[])
 		return 1;
 	}
 	fp = argc > 1 && strcmp(argv[1], "-")? gzopen(argv[1], "r") : gzdopen(fileno(stdin), "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	if (argc > 2) prefix = argv[2];
 
@@ -1444,6 +1537,10 @@ int stk_kfreq(int argc, char *argv[])
 	}
 
 	fp = argc == 2 || strcmp(argv[2], "-") == 0? gzdopen(fileno(stdin), "r") : gzopen(argv[2], "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	ks = kseq_init(fp);
 	while (kseq_read(ks) >= 0) {
 		int k, x[2], cnt[2], cnt_nei[2], which;
@@ -1517,6 +1614,10 @@ int stk_fqchk(int argc, char *argv[])
 		return 1;
 	}
 	fp = (strcmp(argv[optind], "-") == 0)? gzdopen(fileno(stdin), "r") : gzopen(argv[optind], "r");
+	if (fp == 0) {
+		fprintf(stderr, "[E::%s] failed to open the input file/stream.\n", __func__);
+		return 1;
+	}
 	seq = kseq_init(fp);
 	for (k = 0; k <= 93; ++k)
 		perr[k] = pow(10., -.1 * k);
@@ -1574,7 +1675,7 @@ static int usage()
 {
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Usage:   seqtk <command> <arguments>\n");
-	fprintf(stderr, "Version: 1.2-r94\n\n");
+	fprintf(stderr, "Version: 1.3-r106\n\n");
 	fprintf(stderr, "Command: seq       common transformation of FASTA/Q\n");
 	fprintf(stderr, "         comp      get the nucleotide composition of FASTA/Q\n");
 	fprintf(stderr, "         sample    subsample sequences\n");
@@ -1599,28 +1700,27 @@ static int usage()
 int main(int argc, char *argv[])
 {
 	if (argc == 1) return usage();
-	if (strcmp(argv[1], "comp") == 0) stk_comp(argc-1, argv+1);
-	else if (strcmp(argv[1], "fqchk") == 0) stk_fqchk(argc-1, argv+1);
-	else if (strcmp(argv[1], "hety") == 0) stk_hety(argc-1, argv+1);
-	else if (strcmp(argv[1], "gc") == 0) stk_gc(argc-1, argv+1);
-	else if (strcmp(argv[1], "subseq") == 0) stk_subseq(argc-1, argv+1);
-	else if (strcmp(argv[1], "mutfa") == 0) stk_mutfa(argc-1, argv+1);
-	else if (strcmp(argv[1], "mergefa") == 0) stk_mergefa(argc-1, argv+1);
-	else if (strcmp(argv[1], "mergepe") == 0) stk_mergepe(argc-1, argv+1);
-	else if (strcmp(argv[1], "dropse") == 0) stk_dropse(argc-1, argv+1);
-	else if (strcmp(argv[1], "randbase") == 0) stk_randbase(argc-1, argv+1);
-	else if (strcmp(argv[1], "cutN") == 0) stk_cutN(argc-1, argv+1);
-	else if (strcmp(argv[1], "listhet") == 0) stk_listhet(argc-1, argv+1);
-	else if (strcmp(argv[1], "famask") == 0) stk_famask(argc-1, argv+1);
-	else if (strcmp(argv[1], "trimfq") == 0) stk_trimfq(argc-1, argv+1);
-	else if (strcmp(argv[1], "hrun") == 0) stk_hrun(argc-1, argv+1);
-	else if (strcmp(argv[1], "sample") == 0) stk_sample(argc-1, argv+1);
-	else if (strcmp(argv[1], "seq") == 0) stk_seq(argc-1, argv+1);
-	else if (strcmp(argv[1], "kfreq") == 0) stk_kfreq(argc-1, argv+1);
-	else if (strcmp(argv[1], "rename") == 0) stk_rename(argc-1, argv+1);
+	if (strcmp(argv[1], "comp") == 0) return stk_comp(argc-1, argv+1);
+	else if (strcmp(argv[1], "fqchk") == 0) return stk_fqchk(argc-1, argv+1);
+	else if (strcmp(argv[1], "hety") == 0) return stk_hety(argc-1, argv+1);
+	else if (strcmp(argv[1], "gc") == 0) return stk_gc(argc-1, argv+1);
+	else if (strcmp(argv[1], "subseq") == 0) return stk_subseq(argc-1, argv+1);
+	else if (strcmp(argv[1], "mutfa") == 0) return stk_mutfa(argc-1, argv+1);
+	else if (strcmp(argv[1], "mergefa") == 0) return stk_mergefa(argc-1, argv+1);
+	else if (strcmp(argv[1], "mergepe") == 0) return stk_mergepe(argc-1, argv+1);
+	else if (strcmp(argv[1], "dropse") == 0) return stk_dropse(argc-1, argv+1);
+	else if (strcmp(argv[1], "randbase") == 0) return stk_randbase(argc-1, argv+1);
+	else if (strcmp(argv[1], "cutN") == 0) return stk_cutN(argc-1, argv+1);
+	else if (strcmp(argv[1], "listhet") == 0) return stk_listhet(argc-1, argv+1);
+	else if (strcmp(argv[1], "famask") == 0) return stk_famask(argc-1, argv+1);
+	else if (strcmp(argv[1], "trimfq") == 0) return stk_trimfq(argc-1, argv+1);
+	else if (strcmp(argv[1], "hrun") == 0) return stk_hrun(argc-1, argv+1);
+	else if (strcmp(argv[1], "sample") == 0) return stk_sample(argc-1, argv+1);
+	else if (strcmp(argv[1], "seq") == 0) return stk_seq(argc-1, argv+1);
+	else if (strcmp(argv[1], "kfreq") == 0) return stk_kfreq(argc-1, argv+1);
+	else if (strcmp(argv[1], "rename") == 0) return stk_rename(argc-1, argv+1);
 	else {
 		fprintf(stderr, "[main] unrecognized command '%s'. Abort!\n", argv[1]);
 		return 1;
 	}
-	return 0;
 }
